;;; A Common Lisp implementation of the compo protocol Version 2
;;;
;;; Package :compo-cm
;;;
;;; Copyright (C) 2001  Bruno LARTILLOT
;;; 
;;; This program is free software; you can redistribute it and/or
;;; modify it under the terms of the GNU General Public License
;;; as published by the Free Software Foundation; either version 2
;;; of the License, or any later version.
;;;
;;; This program is distributed in the hope that it will be useful,
;;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;;; GNU General Public License for more details.
;;;
;;; You should have received a copy of the GNU General Public License
;;; along with this program; if not, write to the Free Software
;;; Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

(in-package :compo-cm)

(labels ((separ (subs voi resa)
           (if (or (atom subs) (/= (voi (car subs)) voi))
             (cons resa subs)
             (separ (cdr subs) voi (append resa `(,(car subs)))) ) )
 
         (parts (subs)
           (and (consp subs)
                ((lambda (res) (cons (car res)
                                     (parts (cdr res)) ))
                 (separ subs (voi (car subs)) nil) ) ) )
         
         (conv-hei (hei)
           (read (make-string-input-stream
                  (concatenate 'string
                               "cm::"
                               (string-left-trim "(:)"
                                                 (itp hei :syntax :english) ) ) )) )

         (process-part (part f)
           `(cm::new cm::seq :subobjects
                     (list
                      ,@(mapcar
                         #'(lambda (x)
                             `(cm::new cm::midi
                                       :keynum ,(round
                                                 (cm::keynum
                                                  (funcall f (itp (hei x))) ) )
                                       :duration ,(dur x)
                                       :time ,(pos x)
                                       :amplitude
                                       ,(max (min (round (* (dyn x) 100))
                                                  127 )
                                             0 )
                                       :channel
                                       ,(mod (- (voi x)) 16) ) )
                         part ) ) ) ) )

  (defun cmize (note &key (midi-file *default-midi-file*) (ptf #'ptf-eq-440) (tempo 60))

"Function
   cmize (note &key midi-file ptf tempo)
Arguments
   A note structure.
   An output file being a string or a pathname designing a file to be produced,
   defaulting to the current value of the special variable *default-midi-file*.
   A function converting pitches to frequencies, defaulting to the function
   #'ptf-eq-440 (conversion in equal temperament and with a fork tune at 440 Hz).
   A tempo defaulting to 60 (quarter per second).
Values
   No value is returned.
   As a side effect, a Common Music form representing the normalized note structure,
   with one thread per different voice value found in the elementary notes, is produced."

    (and (subs note)
         `(cm::events
           (cm::new cm::seq :subobjects
                    (list
                     ,(set-programs)
                     (cm::new cm::seq :subobjects
                              (list
                               ,@(mapcar #'(lambda (sub)
                                             (process-part sub ptf) )
                                         (parts (subs note)) ) ) ) ) )
           ,(namestring midi-file) :tempo ,tempo ) ) ) )