;;; A Common Lisp implementation of the compo protocol Version 2
;;;
;;; Package :compo-cm
;;;
;;; Copyright (C) 2001  Bruno LARTILLOT
;;; 
;;; This program is free software; you can redistribute it and/or
;;; modify it under the terms of the GNU General Public License
;;; as published by the Free Software Foundation; either version 2
;;; of the License, or any later version.
;;;
;;; This program is distributed in the hope that it will be useful,
;;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;;; GNU General Public License for more details.
;;;
;;; You should have received a copy of the GNU General Public License
;;; along with this program; if not, write to the Free Software
;;; Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

(in-package :compo-cm)

(defvar *instruments* '(0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0))

(defun set-programs ()
  `(cm::new cm::seq :subobjects
            (list
             ,@(let ((count 0))
                 (mapcar
                  #'(lambda (a)
                      (prog1
                        `(cm::new cm::midimsg
                                  :time 0
                                  :msg (cm::make-program-change ,count
                                                                ,a ) )
                        (setf count (1+ count)) ) )
                  *instruments* ) ) ) ) )

(defun init-programs ()

"Function
   init-programs ()
Arguments
   An output file being a string or a pathname designing a file to be produced,
   defaulting to the current value of the special variable *default-midi-file*.
Values
   No value is produced.
   As a side effect, all the midi programs are reinitialized."

  (mapl #'(lambda (a) (setf (car a) 0)) *instruments*)
  *instruments* )

(defun program-change (voi &optional prog)

"Function
   program-change (voi &optional prog)
Arguments
   A midi chanel between 0 and 15.
   A midi program number between 0 and 127.
Values
   No value is produced.
   As a side effect, the specified chanel is set to the specified program number."

  (setf (elt *instruments* (mod (- (cton voi)) 16))
        (let ((hei (check-program-change (or prog :help))))
          (when (eq hei :abort) (abort)) hei ) )
  *instruments* )

(defun test-programs ()

"Function
   test-programs ()
Values
   No value is produced.
   As a side effect, a midi note is sent to each midi chanel."

  (midi (note :32th (:voi 1) (:voi 0) (:voi -1) (:voi -2)
                     (:voi -3) (:voi -4) (:voi -5) (:voi -6)
                     (:voi -7) (:voi -8) (:voi -9) (:voi -10)
                     (:voi -11) (:voi -12) (:voi -13) (:voi -14) ) ) )