;;; A Common Lisp implementation of the compo protocol Version 2
;;;
;;; Package :compo
;;;
;;; Copyright (C) 2001  Bruno LARTILLOT
;;; 
;;; This program is free software; you can redistribute it and/or
;;; modify it under the terms of the GNU General Public License
;;; as published by the Free Software Foundation; either version 2
;;; of the License, or any later version.
;;;
;;; This program is distributed in the hope that it will be useful,
;;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;;; GNU General Public License for more details.
;;;
;;; You should have received a copy of the GNU General Public License
;;; along with this program; if not, write to the Free Software
;;; Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

(in-package :compo)

(defgeneric clone (self)
  (:documentation
"Generic Function
   clone (self)
Arguments
   self is the object to clone.
Values
   A full recursive copy, in some appropriate sense to be defined in each primary method, 
   of the self argument.
Purpose
   Along with 'clone-slots', 'clone' makes copies entirely disjoined from their original." ) )

(defmethod clone ((self t)) 

"Primary method
   clone specialized on 't'
Behaviour
   The value of self is returned. Note that by default, clone doesnt copy its 
   argument (it just doesnt know how to) but simply returns the value to 
   which its argument is currently bound."

  self )

(defmethod clone ((self cons))

"Primary method
   clone specialized on 'cons'
Behaviour
   If self is a circular list on the car (i.e. its car is eq to itself), a new circular
   list on the car, which cdr is initialized to a clone of the cdr of self is returned.
   If self is a circular list on the cdr (i.e. its cdr is eq to itself), a new circular
   list on the cdr, which car is initialized to a clone of the car of self is returned.
   A new cons is returned else, after bounding its car to the result of applying clone 
   to the car of self and its cdr to the result of applying clone to the cdr of self."

  (if (eq (car self) self)
    (circar (clone (cdr self)))
    (if (eq (cdr self) self)
      (circdr (clone (car self)))
      (cons (clone (car self)) (clone (cdr self))) ) ) )

(defmethod clone ((self standard-object))

"Primary method
   clone specialized on 'standard-object'
Behaviour
   'make-instance' is applied to the class of self, using the result of applying
   'clone-slots' to self and the new instanced object, as a remaining argument list (see
   'clone-slots').
   This allow any user class to specialize the cloning process only for the supplementary
   instance slots it defines by simply specializing 'clone-slots' (see this generic
   function)."

   (let ((copy (make-instance (class-of self))))
     (clone-slots self copy)
     copy ) )