;;; A Common Lisp implementation of the compo protocol Version 2
;;;
;;; Package :compo
;;;
;;; Copyright (C) 2001  Bruno LARTILLOT
;;; 
;;; This program is free software; you can redistribute it and/or
;;; modify it under the terms of the GNU General Public License
;;; as published by the Free Software Foundation; either version 2
;;; of the License, or any later version.
;;;
;;; This program is distributed in the hope that it will be useful,
;;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;;; GNU General Public License for more details.
;;;
;;; You should have received a copy of the GNU General Public License
;;; along with this program; if not, write to the Free Software
;;; Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

(in-package :compo)

(defun coerce-value (value coercion-spec)

"Function
   coerce-value (value coercion-spec)
Arguments
   A value to be coerced
   A coercion specifier
Purpose
   This function applies a coercion specifier (i.e. a type specifier returning a value
   rather simply true or false) to a value (see 'coercion' and 'coerce-compo').
Behaviour
   First, if the coercion specifier is an atom, it is normalized to a list of itself.
   If the car of the coercion specifier is a type specifier in the sense defined below,
   the function returns the value itself if it is actually of the type represented by
   that type specifier (i.e. if (typep value coercion-spec) is true), nil otherwise.
   If the car of the coercion specifier is not a type specifier, the function returns the
   result of applying the function represented by the car of the coercion specifier to
   the value, with the cdr of the coercion specifier as a remaining argument list.
   Note : A symbol is considered here to be a type specifier symbol if the type-spec-p
   indicator is true in the property list for that symbol. This is the case (the present
   :compo package especially takes care of that) for any of the Common Lisp standard type
   specifiers symbols (see the table page 50 in Steele : The Language - Second Edition),
   or if the type specifier has been defined using 'mydeftype' rather than the standard
   Common Lisp macro 'deftype' (see mydeftype)."

  (let ((coercion-spec (if (consp coercion-spec)
                         coercion-spec
                         (list coercion-spec) )))
    (if (get (car coercion-spec) 'type-spec-p)
      (and (typep value coercion-spec)
           value )
      (apply (symbol-function (car coercion-spec)) value (cdr coercion-spec)) ) ) )