;;; A Common Lisp implementation of the compo protocol Version 2
;;;
;;; Package :compo
;;;
;;; Copyright (C) 2001  Bruno LARTILLOT
;;; 
;;; This program is free software; you can redistribute it and/or
;;; modify it under the terms of the GNU General Public License
;;; as published by the Free Software Foundation; either version 2
;;; of the License, or any later version.
;;;
;;; This program is distributed in the hope that it will be useful,
;;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;;; GNU General Public License for more details.
;;;
;;; You should have received a copy of the GNU General Public License
;;; along with this program; if not, write to the Free Software
;;; Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

(in-package :compo)

(defun closestc (x &rest l)

"Function
   closestc (x &rest l)
Arguments
   x is a number
   l is a list of numbers
Values
   The value from l which is numerically the closest to x
Purpose
   closestc is a functional coercion specifier, i.e. a function which, when applied to
   a value, possibly along with a parameter list, is expected to return a value of the
   same type or nil. The old value is said to be coerced to the new value by the 
   coercion specifier, nil meaning that the coercion is not possible.
   It is intended to be used along with the function 'coerce-value' (see 'coerce-value')."

  (and l
       (if (atom (cdr l))
         (car l)
         (let ((a (apply #'closestc x (cdr l))))
           (if (< (abs (- x (car l)))
                  (abs (- x a)) )
             (car l)
             a ) ) ) ) )