;;; A Common Lisp implementation of the compo protocol Version 2
;;;
;;; Package :compo
;;;
;;; Copyright (C) 2001  Bruno LARTILLOT
;;; 
;;; This program is free software; you can redistribute it and/or
;;; modify it under the terms of the GNU General Public License
;;; as published by the Free Software Foundation; either version 2
;;; of the License, or any later version.
;;;
;;; This program is distributed in the hope that it will be useful,
;;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;;; GNU General Public License for more details.
;;;
;;; You should have received a copy of the GNU General Public License
;;; along with this program; if not, write to the Free Software
;;; Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

(in-package :compo)

(defclass granulation (context-item)()
  (:documentation

"Class
   granulation
Direct Superclasses
   context-item
Purpose
   This context item is responsible of the normalization of the subcompositions
   present at a given depth. This mechanism allows for instance to momentarily present
   chords (provided that chords figure at the same depth level in a given hierarchy)
   as patterns (C E G for a perfect major chord) in a synthetic purpose, rather than
   according to their current figuration (which could be C3 C4 G4 E5 for instance). " ) )

(defmethod realize-using-context-item ((context-item granulation) 
                                       (compo compo)
                                       &key depth &allow-other-keys)

"Primary method
   realize-using-context-item specialized on the granulation type
Behaviour
   The next method is first called.
   Normally returns the composition unchanged. But if the composition does not have any
   subcomposition, or if depth is provided and equals to 0, the composition is given
   a new subcomposition list which computation is delegated to the generic function
   'granulate-compo' (see 'granulate-compo')."

   (declare (ignore context-item))
   (call-next-method)
   (with-slots (subs) compo
    (unless (and subs
                 (or (not depth)(> depth 0)))
      (granulate-compo compo) ) )
   compo )