;;; A Common Lisp implementation of the compo protocol Version 2
;;;
;;; Package :compo
;;;
;;; Copyright (C) 2001  Bruno LARTILLOT
;;; 
;;; This program is free software; you can redistribute it and/or
;;; modify it under the terms of the GNU General Public License
;;; as published by the Free Software Foundation; either version 2
;;; of the License, or any later version.
;;;
;;; This program is distributed in the hope that it will be useful,
;;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;;; GNU General Public License for more details.
;;;
;;; You should have received a copy of the GNU General Public License
;;; along with this program; if not, write to the Free Software
;;; Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

(in-package :compo)

(defclass selection (context-item)()
  (:documentation

"Class
   selection
Direct Superclasses
   context-item
Purpose
   Base context item provided for the selection of the subcompositions to be keeped." ) )

(defmethod realize-using-context-item ((context-item selection) 
                                       (compo compo)
                                       &rest restargs &key &allow-other-keys)

"Primary method
   realize-using-context-item specialized on the selection type
Behaviour
   The next method is first called.
   The 'select-compo' generic function is applied to each subcomposition with 'restargs'
   as a remaining argument list.
   All the partial results of calling select-compo are appended, the obtained list being
   given as a new value to the 'subs' slot of the current composition.
   'compo' is returned as a result."

   (declare (ignore context-item))
   (call-next-method)
   (setf (subs compo) (mapcan #'(lambda (x) (apply #'select-compo x restargs))
                              (subs compo) ))
   compo )