;;; A Common Lisp implementation of the compo protocol Version 2
;;;
;;; Package :compo
;;;
;;; Copyright (C) 2001  Bruno LARTILLOT
;;; 
;;; This program is free software; you can redistribute it and/or
;;; modify it under the terms of the GNU General Public License
;;; as published by the Free Software Foundation; either version 2
;;; of the License, or any later version.
;;;
;;; This program is distributed in the hope that it will be useful,
;;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;;; GNU General Public License for more details.
;;;
;;; You should have received a copy of the GNU General Public License
;;; along with this program; if not, write to the Free Software
;;; Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

(in-package :compo)

(defun realize (compo &rest restargs 
                      &key (context (context compo))
                           depth &allow-other-keys )

  (declare (ignore depth))

"Function
   realize (compo &rest restargs &key :context :depth &allow-other-keys)
Arguments
   A composition to be realized. 
   restargs is provided as a &rest argument list to give the body an access to the entire
   list of the given key arguments.
   The :context key argument is the context to use in the realization process. It 
   defaults to the current value of the 'default-context' class slot in the composition
   to be realized.
   The :depth key argument should be an integer if provided. If a depth is provided, the
   realization will be a granulation at the specified depth (see 'granulation' and
   'recursion').
Values
   Always returns a valid composition.
Purpose
   This function is the starting point to the realization of compositions. 
   It is callable from anywhere by any client.
   It is non-destructive relatively to the compo and the context arguments.
Behaviour
   The 'nrealize' generic-function is applied to a clone of the composition, the context
   (nrealize is in charge of cloning the context) as a :contexts key argument and
   restargs as a remaining argument list.
   Whatever is returned by 'nrealize' is returned as a result to 'realize'."

  (let ((result (clone compo)))
    (apply #'nrealize
           result
           :context context
           restargs ) ) )