;;; A Common Lisp implementation of the compo protocol Version 2
;;;
;;; Package :compo
;;;
;;; Copyright (C) 2001  Bruno LARTILLOT
;;; 
;;; This program is free software; you can redistribute it and/or
;;; modify it under the terms of the GNU General Public License
;;; as published by the Free Software Foundation; either version 2
;;; of the License, or any later version.
;;;
;;; This program is distributed in the hope that it will be useful,
;;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;;; GNU General Public License for more details.
;;;
;;; You should have received a copy of the GNU General Public License
;;; along with this program; if not, write to the Free Software
;;; Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

(in-package :compo)

(defgeneric coerce-compo (compo coercer &key super-compo prev-compo &allow-other-keys)
  (:documentation

"Generic Function
   coerce-compo (compo coercer &key :super-compo :prev-compo &allow-other-keys)
Purpose
   This generic function is intended to be called only by the realization mechanism
   specialized on the coercion base context item. By providing new primary method, it is
   possible to change the default acception of the coercion in the system, which is the
   filtering of the subcompositions depending on whether or not they are of the same
   class as the corresponding type composition in the coercer list.
   In the case of composition classes defining numeric properties, like the class 'note'
   in the package :note, the coercion mechanism allows to change the value of this
   properties, according to coercion specifiers (see 'coerce-value')." ) )

(defmethod coerce-compo ((compo compo) coercer &key super-compo prev-compo)

"Primary method
   coerce-compo specialized on 'compo'
Behaviour
   If the coercer is null, a list of the composition is returned.
   The coercer is transformed by the received supercomposition, using the generic
   function 'transform'.
   The coercer is then <<evaluated>> in the particular sense defined in the context-item
   'evaluation'.
   If the composition is of the same class as the coercer, a list of itself is returned.
   Else nil is returned."

  (if (not coercer)
    (list compo)
    (progn
      (transform coercer super-compo)
      (evaluate coercer super-compo prev-compo)
      (and (typep compo (class-of coercer))
           (list compo) ) ) ) )