;;; A Common Lisp implementation of the compo protocol Version 2
;;;
;;; Package :compo
;;;
;;; Copyright (C) 2001  Bruno LARTILLOT
;;; 
;;; This program is free software; you can redistribute it and/or
;;; modify it under the terms of the GNU General Public License
;;; as published by the Free Software Foundation; either version 2
;;; of the License, or any later version.
;;;
;;; This program is distributed in the hope that it will be useful,
;;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;;; GNU General Public License for more details.
;;;
;;; You should have received a copy of the GNU General Public License
;;; along with this program; if not, write to the Free Software
;;; Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

(in-package :compo)

(defun read-compo (form)

  (labels ((f (form)
             (and form
                  (cond

                   ; Declaration

                   ((and (cdr form)
                           (symbolp (car form))
                           (not (keywordp (car form)))
                           (eql (position #\= (symbol-name (car form)))
                                (1- (length (symbol-name (car form)))) ) )
                    (let ((a (read-compo (cadr form)))
                          (val (check-name
                                (read-from-string
                                 (string-right-trim "=" (symbol-name (car form))) ) )) )
                      (when (eq val :abort) (abort))
                      (progv `(,val)
                             `(,a)
                        (f (cddr form))
                        ) ) )

                   ((and (cddr form) (eq (cadr form) '=))
                    (let ((a (read-compo (caddr form))))
                      (progv `(,(car form))
                             `(,a)
                        (f (cdddr form))
                        ) ) )

                   ; Multiplication

                   ((and (cdr form)
                         (symbolp (car form))
                         (not (keywordp (car form)))
                         (eql (position #\* (symbol-name (car form)))
                              (1- (length (symbol-name (car form)))) ) )
                    `(,@(do ((it 1 (+ it 1)) (b))
                            ((> it (read-from-string
                                    (string-right-trim "*"
                                                       (symbol-name (car form)) ) ) )
                                b )
                          (setf b (cons (read-compo (cadr form)) b)) )
                      ,@(f (cddr form)) ) )

                   ((and (cddr form) (eq (cadr form) '*))
                    `(,@(do ((it 1 (+ it 1)) (b))
                            ((> it (car form)) b)
                          (setf b (cons (read-compo (caddr form)) b)) )
                      ,@(f (cdddr form)) ) )

                   ; Reference

                   ((and (symbolp (car form)) (not (keywordp (car form))))
                    (cons (symbol-value (let ((val (check-reference (car form))))
                                          (when (eq val :abort) (abort)) val ))
                          (f (cdr form))) )

                   ; Anonymous simple value

                   (t
                    (cons (if (atom (car form)) (car form) (read-compo (car form)))
                          (f (cdr form)) ) ) ) ) ))

    (if (or (not form)
            (not (cdr form)) )
      form
      (if (not (eq (car form) :subs))
        (cons (car form) (read-compo (cdr form)))
        (if (and (consp (cadr form)) (eq (caadr form) 'list))
          (list (car form) (cons 'list (f (cdadr form))))
          (cons (car form) (f (cdr form))) ) ) ) ) )