;;; A Common Lisp implementation of the compo protocol Version 2
;;;
;;; Package :compo
;;;
;;; Copyright (C) 2001  Bruno LARTILLOT
;;; 
;;; This program is free software; you can redistribute it and/or
;;; modify it under the terms of the GNU General Public License
;;; as published by the Free Software Foundation; either version 2
;;; of the License, or any later version.
;;;
;;; This program is distributed in the hope that it will be useful,
;;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;;; GNU General Public License for more details.
;;;
;;; You should have received a copy of the GNU General Public License
;;; along with this program; if not, write to the Free Software
;;; Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

(in-package :compo)

(defmethod print-object ((self abstract-compo) stream)

"Primary-method
   print-object specialized on 'compo'
Behaviour
   An opening parenthesis starts the printing of the composition, and a closing
   parenthesis ends it.
   The class name of the composition is printed, before delegating the printing of the
   eventual properties defined by any subclass of compo to the generic function
   'specialized-print' and the printing of each subcomposition to the function 'sub-print',
   taking care of the two following special cases :
   . If the value of the slot 'subs' in the composition is not a list, it represents the
     particular case of an atomic subcomposition, which is indicated by making its
     representation preceded by the keyword :subs. Note that this is also the way to
     specify this particular case when entering a new composition (see normalize-compo).
   . In that case of an atomic subcomposition, if the subcomposition represents the
     composition itself, it is represented by the keyword :self to avoid any circularity
     when printing."

  (write-string "(" stream)
  (write (class-name (class-of self)) :stream stream)
  (specialized-print self stream t)
  (when (subs self)
    (write-string " " stream)
    (if (eq (subs self) self)
      (write :self :stream stream)
      (if (atom (subs self))
        (progn
          (write-string ":SUBS " stream)
          (sub-print (subs self) self stream) )
        (let ((sp))
          (mapl #'(lambda (x)
                    (when sp (write-string " " stream))
                    (sub-print (car x) self stream)
                    (setf sp t) )
                (subs self) ) ) ) ) )
  (write-string ")" stream) )