;;; A Common Lisp implementation of the compo protocol Version 2
;;;
;;; Package :compo
;;;
;;; Copyright (C) 2001  Bruno LARTILLOT
;;; 
;;; This program is free software; you can redistribute it and/or
;;; modify it under the terms of the GNU General Public License
;;; as published by the Free Software Foundation; either version 2
;;; of the License, or any later version.
;;;
;;; This program is distributed in the hope that it will be useful,
;;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;;; GNU General Public License for more details.
;;;
;;; You should have received a copy of the GNU General Public License
;;; along with this program; if not, write to the Free Software
;;; Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

(in-package :compo)

(export '(*comment-descr*
          *coercer-specifier-name-descr*
          *operator-name-descr*
          *common-lisp-number-descr*
          *symbol-descr*
          *invalid-name-error*
          *unbound-reference-error*
          *hei-keyword-error*
          *dur-keyword-error*
          *pos-mod-keyword-error*
          *pos-keyword-error*
          *voi-keyword-error*
          *dyn-keyword-error*
          *hei-coercer-error*
          *dur-coercer-error*
          *pos-coercer-error*
          *voi-coercer-error*
          *dyn-coercer-error*
          *parameter-key-error*
          *program-change-error*
          *cleave-change-error*
          *err-prompt*
          *err2-prompt*
          *help-prompt* ))

(defparameter *comment-descr*
  "Any string.")

(defparameter *coercer-specifier-name-descr*
  "The name of any Common Lisp type specifier, any user type specifier defined using the 
function mydeftype, or any function acting as a coercer specifier in the sense defined in
compo.")

(defparameter *operator-name-descr*
  "The name of any function of n-argument applying exclusively on numbers.")

(defparameter *common-lisp-number-descr*
  "Whatever value v such (typep v 'number) is true." )

(defparameter *symbol-descr*
  "Any symbol which is not one of 'note 'cnote 'fermata 'repeat-end 'fermata-repeat-end and 'lyrics." )

(defparameter *invalid-name-error*
  "~S is not a valid name !")

(defparameter *unbound-reference-error*
  "~S does not reference a previously declared name !")

(defparameter *hei-keyword-error*
  "~S is not a valid musical height !")

(defparameter *dur-keyword-error*
  "~S is not a valid musical duration !")

(defparameter *pos-mod-keyword-error*
  "~S is not a valid position mode !")

(defparameter *pos-keyword-error*
  "~S is not a valid position !")

(defparameter *voi-keyword-error*
  "~S is not a valid musical voice !")

(defparameter *dyn-keyword-error*
  "~S is not a valid musical dynamics value !")

(defparameter *hei-coercer-error*
  "~S is not a valid height coercer !")

(defparameter *dur-coercer-error*
  "~S is not a valid duration coercer !")

(defparameter *pos-coercer-error*
  "~S is not a valid position coercer !")

(defparameter *voi-coercer-error*
  "~S is not a valid voice coercer !")

(defparameter *dyn-coercer-error*
  "~S is not a valid dynamics coercer !")

(defparameter *parameter-key-error*
  "~S is not a valid parameter keyword !")

(defparameter *program-change-error*
  "~S is not a valid midi program !")

(defparameter *cleave-change-error*
  "~S is not a valid cmn cleave !")

(defparameter *err-prompt*
  "Please, enter a valid value, :help for a description of valid values, or :abort ? ")

(defparameter *err2-prompt*
  "Please, enter a valid value or :abort ? ")

(defparameter *help-prompt*
  "Please, enter :
  :abort to quit the help
  t for a general description of a note form
  :hei for a description of valid height values
  :dur for a description of valid duration values
  :pos-mod for a description of valid position modes
  :pos for a description of valid position values
  :voi for a description of valid voice values
  :dyn for a description of valid dynamics values
  :parameter-key for a description of valid parameter keywords
  :subs for a description of subnote forms
  :coerce-hei for a description of valid coercer specifiers on heights
  :coerce-dur for a description of valid coercer specifiers on durations
  :coerce-pos for a description of valid coercer specifiers on positions
  :coerce-voi for a description of valid coercer specifiers on voices
  :coerce-dyn for a description of valid coercer specifiers on dynamics
<help> ? ")