;;; A Common Lisp implementation of the compo protocol Version 2
;;;
;;; Package :note
;;;
;;; Copyright (C) 2001  Bruno LARTILLOT
;;; 
;;; This program is free software; you can redistribute it and/or
;;; modify it under the terms of the GNU General Public License
;;; as published by the Free Software Foundation; either version 2
;;; of the License, or any later version.
;;;
;;; This program is distributed in the hope that it will be useful,
;;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;;; GNU General Public License for more details.
;;;
;;; You should have received a copy of the GNU General Public License
;;; along with this program; if not, write to the Free Software
;;; Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

(in-package :note)

(defun cleavep (x)

"Predicate
   cleavep (x)
Arguments
   A value to be considered or not as being a cleave
Values
   The argument if it is a keyword belonging to the list
      :french-violin
      :treble
      :soprano
      :mezzo-soprano
      :alto
      :tenor
      :baritone
      :bass
      :double-bass
      :percussion
   nil otherwise."

  (and
   (member x '(:french-violin
               :treble
               :soprano
               :mezzo-soprano
               :alto
               :tenor
               :baritone
               :bass
               :sub-bass
               :double-bass
               :percussion ))
   x ) )

(defun ntoc (num)

"Function
   ntoc (num)
Arguments
   An integer to be translated to a cleave keyword
Values
   The keyword or nil if the integer doesn't correspond to a known keyword
Behaviour
   The correspondance is as the following :
       1 :french-violin
       0 :treble
      -1 :soprano
      -2 :mezzo-soprano
      -3 :alto
      -4 :tenor
      -5 :baritone
      -6 :bass
      -7 :sub-bass
      -8 :double-bass
      -9 :percussion
   Note that this numerotation takes into account that the treble clef represents the
   neutral value (0) for voices (which means that when no particular clef is specified, a
   composition is represented in treble clef), and that voices are intended to appear
   from the bass to the treble in a realized composition (assuming that voices are 
   ordered by ascending voices)."

 (cond 
  ((= num 1) :french-violin)
  ((= num 0) :treble)
  ((= num -1) :soprano)
  ((= num -2) :mezzo-soprano)
  ((= num -3) :alto)
  ((= num -4) :tenor)
  ((= num -5) :baritone)
  ((= num -6) :bass)
  ((= num -7) :sub-bass)
  ((= num -8) :double-bass)
  ((= num -9) :percussion)
  (t nil) ) )

(defun ntoev (num)

"Function
   ntoev (num)
Arguments
   An integer to be translated to an elementary voice
Values
   The elementary voice (between -14 and 1) corresponding to the argument."

  (multiple-value-bind (q r) (floor (+ num 14) 16)
    (values (- (mod r 16) 14) q) ) )

(defun cton (clef)

"Function
   cton (clef)
Arguments
   A cleave keyword to be translated to a number
Values
   The number or the argument if the cleave keyword doesn't correspond to a known keyword
Behaviour
   The correspondance is as the following :
      :french-violin  1
      :treble  0
      :soprano -1
      :mezzo-soprano -2
      :alto -3
      :tenor -4
      :baritone -5
      :bass -6
      :sub-bass -7
      :double-bass -8
      :percussion -9"

 (cond 
  ((eq clef :french-violin) 1)
  ((eq clef :treble) 0)
  ((eq clef :soprano) -1)
  ((eq clef :mezzo-soprano) -2)
  ((eq clef :alto) -3)
  ((eq clef :tenor) -4)
  ((eq clef :baritone) -5)
  ((eq clef :bass) -6)
  ((eq clef :sub-bass) -7)
  ((eq clef :double-bass) -8)
  ((eq clef :percussion) -9)
  (t clef) ) )