;;; A Common Lisp implementation of the compo protocol Version 2
;;;
;;; Package :note
;;;
;;; Copyright (C) 2001  Bruno LARTILLOT
;;; 
;;; This program is free software; you can redistribute it and/or
;;; modify it under the terms of the GNU General Public License
;;; as published by the Free Software Foundation; either version 2
;;; of the License, or any later version.
;;;
;;; This program is distributed in the hope that it will be useful,
;;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;;; GNU General Public License for more details.
;;;
;;; You should have received a copy of the GNU General Public License
;;; along with this program; if not, write to the Free Software
;;; Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

(in-package :note)

(defmethod coerce-compo ((note note) coercer &key &allow-other-keys)

"Primary method
   coerce-compo specialized on the note type
Purpose
   1) If the result of calling the next method is not null, it is to be returned by this
   method provided that none of the coercions made in the step described in 2
   returns a null value. If it is the case, the method returns nil.
   2) The values of the slots 'hei', 'voi', 'pos', 'dur' and 'dyn' from the note are
   respectively set to the result of submitting themselves to their respective coercion
   specifier in the coercer, provided that this result is not null."

  (let ((a (call-next-method)))
    (if (not coercer) 
      (and (hei note) a)
      (and
       a
       (hei note)
       (let ((hei (and (hei coercer)
                       (coerce-value (hei note) (cadr (hei coercer))) )))
         (when hei
           (setf (hei note) hei)
           hei ) )
       (let ((voi (and (voi coercer)
                       (coerce-value (voi note) (cadr (voi coercer))) )))
         (when voi
           (setf (voi note) voi)
           voi ) )
       (let ((pos (and (pos coercer)
                       (coerce-value (pos note) (cadr (pos coercer))) )))
         (when pos
           (setf (pos note) pos)
           pos ) )
       (let ((dur (and (dur coercer)
                       (coerce-value (dur note) (cadr (dur coercer))) )))
         (when dur
           (setf (dur note) dur)
           dur ) )
       (let ((dyn (and (dyn coercer)
                       (coerce-value (dyn note) (cadr (dyn coercer))) )))
         (when dyn
           (setf (dyn note) dyn)
           dyn ) )
       a ) ) ) )